from sklearn.cluster import KMeans
import pandas as pd
import socket
from collections import OrderedDict
import time

HOST = '127.0.0.1'
PORT = 65432

devices = OrderedDict({'fridge_load': '192.168.1.1', 'freezer1_load': '192.168.1.2'})


class Freezer:
    def send_data(self, ip=devices['fridge_load'], code='0'):
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
            s.connect((HOST, PORT))
            message = ip + ',' + code
            s.sendall(str.encode(message))


class Fridge:
    def send_data(self, ip=devices['freezer1_load'], code='74'):
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
            s.connect((HOST, PORT))
            message = ip + ',' + code
            s.sendall(str.encode(message))


class DBServer:
    def __init__(self, initial_dataset='PATH_TO_DATA'):
        self.da_server = DAServer()
        data = pd.read_csv(initial_dataset)
        self.da_server.train(data)

    def save_data(self, data):
        # saving the data in the DB
        pass

    def get_data(self, data):
        prediction = self.da_server.predict(data)
        if prediction:
            self.save_data(prediction)

    def listen(self):
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
            s.bind((HOST, PORT))
            s.listen()
            conn, addr = s.accept()
            with conn:
                print('Connected by', addr)

                while True:
                    data = conn.recv(1024)
                    if data:
                        conn.sendall(data)
                        ip, code = repr(data)

                        self.get_data(data)


class DAServer:
    def __init__(self):
        self.algorithm = KMeans(n_clusters=2)

    def train(self, data):
        self.algorithm.fit(data)

    def predict(self, data):
        return self.algorithm.predict(data)


if __name__ == '__main__':
    fridge = Fridge()
    freezer = Freezer()
    server = DBServer('PATH_TO_INITIAL_DATASET')

    server.listen()
